FusionHUD = LibStub("AceAddon-3.0"):NewAddon("FusionHUD", "AceConsole-3.0", "AceEvent-3.0");
local AceGUI = LibStub("AceGUI-3.0");

local _G = _G;
FusionHUDLastUpdateClock = GetTime();
 --Set up FusionHUD Options (Ace3 Config)
FusionHUDOptions = {
    name = "Fusion HUD",
    handler = FusionHUD,
	childGroups = "tree",
    type = "group",
    args = {
		lockedWarning = {
			name = "FusionHUD has unlocked elements.  Please use the lock panel at the top of the screen to re-lock the HUD. If you do not see the lock panel, please press the button below to show it, and then lock FusionHUD using the lock button.",
			type = "description",
			order = 1,
			hidden = function() return not FusionHUDModSettings.isUnlocked end,
			fontSize = "medium",
		},
		showLockFrame = {
			name = "Show Lock Panel",
			type = "execute",
			hidden = function() return not FusionHUDModSettings.isUnlocked end,
			order = 2,
			width = "full",
			desc = "Shows the FusionHUD Lock Panel",
			func = 	function() FusionHUDLockFrame:Show(); InterfaceOptionsFrame:Hide(); end,
		}, 
		newBar = {
			name = "New Bar",
			type = "execute",
			desc = "Creates a new HUD bar",
			hidden = function() return FusionHUDModSettings.isUnlocked end,
			order = 1,
			width = "normal",
			func = 		function()
							local index = 0;
							table.foreach(FusionHUDBars, function(k,v) if index < v.barIndex then index = v.barIndex end end);
							FusionHUDBars["bar" .. index + 1] = FusionHUDBarPrototype:New(index + 1)
						end,
		},
		deleteBar = {
			name = "Delete Bar",
			order = 2,
			type = "execute",
			hidden = function() return FusionHUDModSettings.isUnlocked end,
			desc = "Deletes the last bar created",
			width = "normal",
			confirm = function() return "Are you sure? This cannot be undone" end,
			func =	function()
						FusionHUD:DeleteBar() 
					end	
		},
		unlockAll = {
			name = "Unlock All Bars",
			type = "execute",
			desc = "Unlocks all HUD bars for repositioning",
			hidden = function() return FusionHUDModSettings.isUnlocked end,
			order = 4,
			width = "double",
			func = 	function() 
						table.foreach(FusionHUDBars, 	function(k,v)
															v.draggable = true;
															FusionHUD:LockHandler(v);
															FusionHUDModSettings.isUnlocked = true;
														end);
						if FusionHUDModSettings.classBarToggle then								
							FusionHUDClassBar.draggable = true;
							FusionHUD:LockHandler(FusionHUDClassBar);
						end
						FusionHUDModSettings.isUnlocked = true,
						FusionHUDLockFrame:Show();
						InterfaceOptionsFrame:Hide();
					end,
		},
		mainOptions = {
			name = "Main",
			type = "group",
			order = 20,
			hidden = function() return FusionHUDModSettings.isUnlocked end,
			args = {
				transparency = {
					name = "Transparency Settings",
					type = "group",
					order = 7,
					args = {
						contextToggle = {
							name = "Context-sensitive Visibility",
							type = "toggle",
							order = 1,
							desc = "Enables changing HUD visibility based on player situation",
							descStyle = "inline",
							get = 	function() return FusionHUDModSettings.multiTransparency end,
							set =	function() FusionHUDModSettings.multiTransparency = not FusionHUDModSettings.multiTransparency end,
						},
						contextOffTransparency = {
							name = "Transparency",
							type = "range",
							desc = "Set the HUD's overall transparency",
							order = 5,
							hidden = function() return FusionHUDModSettings.multiTransparency end,
							min = 0,
							max = 100,
							step = 1,
							get = function() return FusionHUDModSettings.transparency*100 end,
							set = function(k,v) FusionHUDModSettings.transparency = v/100 end,
						},
						contextOnHiVis = {
							name = "Maximum visibility",
							type = "range",
							desc = "Set the HUD visibility while in combat",
							order = 10,
							hidden = function() return not FusionHUDModSettings.multiTransparency end,
							min = 0,
							max = 100,
							step = 1,
							get = function() return FusionHUDModSettings.hiVis*100 end,
							set = function(k,v) FusionHUDModSettings.hiVis = v/100 end,
						},
						contextOnMidVis = {
							name = "Maximum visibility",
							type = "range",
							desc = "Shows when in combat",
							order = 10,
							hidden = function() return not FusionHUDModSettings.multiTransparency end,
							min = 0,
							max = 100,
							step = 1,
							get = function() return FusionHUDModSettings.midVis*100 end,
							set = function(k,v) FusionHUDModSettings.midVis = v/100 end,
						},
						contextOnMidVis = {
							name = "Medium visibility",
							type = "range",
							desc = "Shows when: targeting, hurt, needing power and not in combat",
							order = 12,
							hidden = function() return not FusionHUDModSettings.multiTransparency end,
							min = 0,
							max = 100,
							step = 1,
							get = function() return FusionHUDModSettings.midVis*100 end,
							set = function(k,v) FusionHUDModSettings.midVis = v/100 end,
						},
						contextOnLoVis = {
							name = "Minimum visibility",
							type = "range",
							desc = "Shows when out of combat, not targeting, not debuffed, at full health/power",
							order = 14,
							hidden = function() return not FusionHUDModSettings.multiTransparency end,
							min = 0,
							max = 100,
							step = 1,
							get = function() return FusionHUDModSettings.loVis*100 end,
							set = function(k,v) FusionHUDModSettings.loVis = v/100 end,
						},
					},					
				},
				hideMinimapIcon = {
					name = "Hide minimap button",
					type = "toggle",
					desc = "Hides the FusionHUD minimap button",
					order = 14,
					get = 	function() return FusionHUDModSettings.hideFusionHUDMinimapIcon end,
					set = 	function()
								FusionHUDModSettings.hideFusionHUDMinimapIcon =  not FusionHUDModSettings.hideFusionHUDMinimapIcon;
								if FusionHUDModSettings.hideFusionHUDMinimapIcon then
									FusionHUDMinimapButtonFrame:Hide();
								else
									FusionHUDMinimapButtonFrame:Show();
								end
							end,
				},
				hideBlizzPlayerFrame = {
					name = "Hide Blizzard player frame",
					type = "toggle",
					desc = "Hides the Blizzard player frame",
					order = 15,
					disabled = function() 
								if PlayerFrame:IsVisible() == false and FusionHUDModSettings.hideBlizzPlayerFrame ==  false then
									return true 
								else 
									return false 
								end
							end,
					get = 	function() return FusionHUDModSettings.hideBlizzPlayerFrame end,
					set = 	function() 
								FusionHUDModSettings.hideBlizzPlayerFrame =  not FusionHUDModSettings.hideBlizzPlayerFrame;
								if FusionHUDModSettings.hideBlizzPlayerFrame then
									PlayerFrame:SetScript("OnEvent", nil);
									PlayerFrame:Hide();
								else
									PlayerFrame:SetScript("OnEvent", PlayerFrame_OnEvent);
									PlayerFrame:Show();
								end
							end,
				},	
				hideBlizzPetFrame = {
					name = "Hide Blizzard player pet frame",
					type = "toggle",
					desc = "Hides the Blizzard player pet frame",
					order = 16,
					disabled = function() 
								if PetFrame:IsVisible() == false and FusionHUDModSettings.hideBlizzPetFrame ==  false then
									return true 
								else 
									return false 
								end
							end,
					get = 	function() return FusionHUDModSettings.hideBlizzPetFrame end,
					set = 	function() 
								FusionHUDModSettings.hideBlizzPetFrame =  not FusionHUDModSettings.hideBlizzPetFrame;
								if FusionHUDModSettings.hideBlizzPetFrame then
									PetFrame:SetScript("OnEvent", nil);
									PetFrame:Hide();
								else
									PetFrame:SetScript("OnEvent", PetFrame_OnEvent);
									PetFrame:Show();
								end
							end,
				},	
				hideBlizzTargetFrame = {
					name = "Hide Blizzard target frame",
					type = "toggle",
					desc = "Hides the Blizzard target frame",
					order = 17,
					disabled = function() 
								if TargetFrame:IsVisible() == false and FusionHUDModSettings.hideBlizzTargetFrame ==  false then
									return true 
								else 
									return false 
								end
							end,
					get = 	function() return FusionHUDModSettings.hideBlizzTargetFrame end,
					set = 	function() 
								FusionHUDModSettings.hideBlizzTargetFrame =  not FusionHUDModSettings.hideBlizzTargetFrame;
								if FusionHUDModSettings.hideBlizzTargetFrame then
									TargetFrame:SetScript("OnEvent", nil);
									TargetFrame:Hide();
								else
									TargetFrame:SetScript("OnEvent", PlayerFrame_OnEvent);
									TargetFrame:Show();
								end
							end,
				},	
			},
		},
		classBar = {
			name = "Class Bar",
			type = "group",
			order = 30,
			hidden = function() return FusionHUDModSettings.isUnlocked end,
			desc = "Settings for the class-specific bar",
			args = {
				classBarHeader = {
					name = "The class bar is currently only available to Warlocks (soul shards), Paladins (holy power), Rogues,(combo points), Druids (combo points and eclipse bar), and Death Knights (runes)",
					type = "description",
					order = 1,
					fontSize = "medium",
				},					
				classBarToggle = {
					name = "Enable Class bar",
					type = "toggle",
					order = 5,
					disabled = function() 
						local localizedClass, englishClass = UnitClass("player"); 
						if englishClass ~= "DEATHKNIGHT" and englishClass ~= "PALADIN" and englishClass ~= "WARLOCK" and englishClass ~= "ROGUE" and englishClass ~= "DRUID" then 
							return true 
						else 
							return false 
						end 
					end,
					desc = "Enables/Disables Class-specific bar (Only available for Death Knights, Paladins and Warlocks)",
					get = 	function() return FusionHUDModSettings.classBarToggle end,
					set =	function() 
								FusionHUDModSettings.classBarToggle = not FusionHUDModSettings.classBarToggle;
								if FusionHUDModSettings.classBarToggle then
									FusionHUDClassBar.frame:Show()
								else
									FusionHUDClassBar.frame:Hide()
								end
							end,
				},					
				classBarSize = {
					name = "Class bar scale",
					type = "range",
					desc = "Controls the size of the class-specific bar",
					disabled = function() return not FusionHUDModSettings.classBarToggle end,
					min = 0,
					max = 150,
					step = 1,
					get = 	function() return FusionHUDClassBar.frame:GetHeight() end,
					set = 	function(k,v) 
								local i;
								local localizedClass, englishClass = UnitClass("player");
								FusionHUDClassBar.height = v;
								FusionHUDClassBar.width = v * 6;
								if englishClass == "ROGUE" then
									FusionHUDClassBar.width = v;
									FusionHUDClassBar.frame.comboFrame.text:SetFont("Interface\\AddOns\\FusionHUD\\Fonts\\DroidSans.ttf", math.floor(FusionHUDClassBar.frame:GetHeight() * .8), "OUTLINE");
								end
								FusionHUDClassBar.frame:SetHeight(FusionHUDClassBar.height); 
								FusionHUDClassBar.frame:SetWidth(FusionHUDClassBar.width);
								if englishClass == "DRUID" then
									local name, tex, tier, col, rank = GetTalentInfo(3, 21, false, false, nil);
									local form = GetShapeshiftForm(false);
									if form == 3 then
										FusionHUDClassBar.frame:SetHeight(FusionHUDClassBar.height);
										FusionHUDClassBar.frame:SetWidth(FusionHUDClassBar.height);
									elseif form == 5 and rank == 1 then
										FusionHUDClassBar.frame:SetHeight(FusionHUDClassBar.height);
										FusionHUDClassBar.frame:SetWidth(FusionHUDClassBar.width);
									end
									FusionHUDClassBar.frame.comboFrame.text:SetFont("Interface\\AddOns\\FusionHUD\\Fonts\\DroidSans.ttf", math.floor(FusionHUDClassBar.frame:GetHeight() * .8), "OUTLINE");
									FusionHUDClassBar.frame.eclipseFrame.arrowFrame:SetHeight(FusionHUDClassBar.height * 4 / 5);
									FusionHUDClassBar.frame.eclipseFrame.arrowFrame:SetWidth(FusionHUDClassBar.height * .3);
								end
								if englishClass == "DEATHKNIGHT" then
									for i = 1,6 do
										FusionHUDClassBar.frame.runes[i]:SetWidth(v);
									end
								elseif englishClass == "PALADIN" then
									FusionHUD:UNIT_POWER("UNIT_POWER", "player", "HOLY_POWER");
								elseif englishClass == "WARLOCK" then
									FusionHUD:UNIT_POWER("UNIT_POWER", "player", "SOUL_SHARDS");
								elseif englishClass == "ROGUE" or englishClass == "DRUID" then
									FusionHUD:UNIT_COMBO_POINTS("UNIT_COMBO_POINTS");
								end
							end,
				},
				classBarUnlock = {
					name = "Unlock Class bar",
					type = "execute",
					order = 15,
					disabled = function() return not FusionHUDModSettings.classBarToggle end,
					desc = "Unlocks the class bar for dragging.  Use the Lock button at the top of the screen when done.",
					func = 	function()
								FusionHUDClassBar.draggable = true;
								FusionHUD:LockHandler(FusionHUDClassBar);
								FusionHUDLockFrame:Show();
								FusionHUDModSettings.isUnlocked = true,
								InterfaceOptionsFrame:Hide();
							end,
				},
			},
		},		
		bars = {
			name = "Bar List",
			order = 50,
			type = "group",
			hidden = function() return FusionHUDModSettings.isUnlocked end,
			desc = "Listing of each HUD bar",
			args = {}
		},
    },
};

function FusionHUD:OnInitialize()	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("FusionHUD", FusionHUDOptions);
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FusionHUD", "FusionHUD");
	--slash commands for opening options
	SLASH_FUSIONHUD1, SLASH_FUSIONHUD2 = '/fusionhud', '/fh';
end
--Build the base frames for the addon, Master frame and Lock Frame
function FusionHUD:BuildBaseFrames()
	FusionHUDMasterFrame = CreateFrame("Frame", nil, UIParent);
	FusionHUDMasterFrame:SetClampedToScreen(true);
	FusionHUDMasterFrame:SetPoint("CENTER",UIParent,"CENTER");
	FusionHUDMasterFrame:SetFrameStrata("LOW");
	FusionHUDMasterFrame:SetHeight(400);
	FusionHUDMasterFrame:SetWidth(400);
	FusionHUDMasterFrame:Show();
	FusionHUDMasterFrame:SetAlpha(1);

	FusionHUDLockFrame = CreateFrame("Frame", nil, UIParent);
	FusionHUDLockFrame:SetClampedToScreen(true);
	FusionHUDLockFrame:ClearAllPoints();
	FusionHUDLockFrame:SetPoint("TOP", UIParent, "TOP", 0, -90);
	FusionHUDLockFrame:SetFrameStrata("FULLSCREEN");
	FusionHUDLockFrame:SetHeight(100);
	FusionHUDLockFrame:SetWidth(200);
	FusionHUDLockFrame.texture = FusionHUDLockFrame:CreateTexture(nil, "BACKGROUND");
	FusionHUDLockFrame.texture:SetAllPoints(FusionHUDLockFrame);
	FusionHUDLockFrame.texture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\whitebackdropbase", true);
	FusionHUDLockFrame.texture:SetVertexColor(0.7, 0.1, 0.1, 1);
	FusionHUDLockFrame.texture:SetTexCoord(0,1,0,1)
	FusionHUDLockFrame.text1 = FusionHUDLockFrame:CreateFontString(nil, "HIGH")
	FusionHUDLockFrame.text1:SetPoint("TOP", FusionHUDLockFrame, "TOP",0, -7);
	FusionHUDLockFrame.text1:SetFont("Fonts\\FRIZQT__.TTF", 24, "OUTLINE");
	FusionHUDLockFrame.text1:SetJustifyH("CENTER");
	FusionHUDLockFrame.text1:SetJustifyV("TOP");
	FusionHUDLockFrame.text1:SetHeight(30);
	FusionHUDLockFrame.text1:SetWidth(150);
	FusionHUDLockFrame.text1:SetText("FusionHUD");
	FusionHUDLockFrame.text2 = FusionHUDLockFrame:CreateFontString(nil, "HIGH")
	FusionHUDLockFrame.text2:SetPoint("TOP", FusionHUDLockFrame.text1, "BOTTOM",0, 0);
	FusionHUDLockFrame.text2:SetFont("Interface\\AddOns\\FusionHUD\\Fonts\\DroidSans.ttf", 14);
	FusionHUDLockFrame.text2:SetJustifyH("CENTER");
	FusionHUDLockFrame.text2:SetJustifyV("TOP");
	FusionHUDLockFrame.text2:SetHeight(30);
	FusionHUDLockFrame.text2:SetWidth(150);
	FusionHUDLockFrame.text2:SetText("Use this button to relock your bars");
	FusionHUDLockFrame:Hide();
	
	FusionHUDLockFrame.lockButton = CreateFrame("Button", "lockButton", FusionHUDLockFrame, "UIPanelButtonTemplate")
	FusionHUDLockFrame.lockButton:SetPoint("TOP", FusionHUDLockFrame.text2, "BOTTOM", 0, 0);
	FusionHUDLockFrame.lockButton:SetWidth(100);
	FusionHUDLockFrame.lockButton:SetHeight(30);
	FusionHUDLockFrame.text1:SetVertexColor(1,1,.4,1);
	FusionHUDLockFrame.text2:SetVertexColor(1, 1, 1, 1);
	FusionHUDLockFrame.lockButton:SetText("Lock");
	FusionHUDLockFrame.lockButton:SetFrameStrata("FULLSCREEN_DIALOG");
end

-- slash command Handler
function SlashCmdList.FUSIONHUD(msg, editbox)
	InterfaceOptionsFrame_OpenToCategory(FusionHUD.optionsFrame);
end

function FusionHUD:OnEnable()
	local name = UnitName("player");
	FusionHUDBars = {};
	FusionHUD:BuildBaseFrames();
	if FusionHUDModSettings.hideFusionHUDMinimapIcon == nil then
		FusionHUDModSettings.hideFusionHUDMinimapIcon = false;
	end
	if FusionHUDModSettings.hideFusionHUDMinimapIcon then
		FusionHUDMinimapButtonFrame:Hide();
	else
		FusionHUDMinimapButtonFrame:Show();
	end
--Get saved settings and populate the bars with them, ensure variables are saved when exiting WOW
	if FusionHUDDB then
		if FusionHUDDB[name] then
			if FusionHUDDB[name].settings then
				table.foreach(FusionHUDDB[name].settings, function(k,v)
					FusionHUDModSettings[k] = FusionHUDDB[name].settings[k]
				end);
			end
			if FusionHUDDB[name].bars then
				table.foreach(FusionHUDDB[name].bars, function(k,v)
					FusionHUDBars[k] = FusionHUDDB[name].bars[k];
					setmetatable(FusionHUDBars[k], FusionHUDBarPrototype);
					FusionHUDBars[k].lastValue = -99999999999;
					FusionHUDBars[k].lastValueMax = -99999999999;
					FusionHUDBars[k]:BuildBar();
				end);
			end
			if FusionHUDDB[name].classBar then
				table.foreach(FusionHUDDB[name].classBar, function(k,v)
					FusionHUDClassBar[k] = FusionHUDDB[name].classBar[k];
				end);
			end
		else
			FusionHUDDB[name] = {};
		end
	else
		FusionHUDDB = {};
		FusionHUDDB[name] = {};
	end
	FusionHUDDB[name].settings = FusionHUDModSettings;
	FusionHUDDB[name].bars = FusionHUDBars;
	FusionHUDDB[name].classBar = FusionHUDClassBar;	
--Start Class Bar setup
	FusionHUD:BuildClassBarFrame()
	FusionHUD:ClassSelector()
--Establish OnUpdate script for Master frame
	FusionHUDMasterFrame:SetScript("OnUpdate", 	function() FusionHUD:OnUpdate()	end); 
--Establish Lock Button behavior	
	FusionHUDLockFrame.lockButton:SetScript('OnClick', 	function() 
		FusionHUDModSettings.isUnlocked = false;
		table.foreach(FusionHUDBars, function(k,v) 
			FusionHUDBars[k].draggable = false;
			FusionHUD:LockHandler(v);
			FusionHUDModSettings.isUnlocked = false;
		end);
		FusionHUDClassBar.draggable = false;
		FusionHUD:LockHandler(FusionHUDClassBar);																							
	end);
	
end	

function FusionHUD:OnUpdate()
	local FusionHUDNextUpdateClock = GetTime()
	local localizedClass, englishClass = UnitClass("player");
--Updates frames every 1/10th of a second unless no change or forced
	if FusionHUDNextUpdateClock - FusionHUDLastUpdateClock > 0.1 then
		FusionHUDLastUpdateClock = FusionHUDNextUpdateClock;
		table.foreach(FusionHUDBars, 	function(k,v) 
											FusionHUDBars[k]:Update() 
										end);
		if englishClass == "DRUID" and FusionHUDModSettings.classBarToggle == true then
			if FusionHUDModSettings.isUnlocked then
				FusionHUDMasterFrame:SetAlpha(1);
				if form == 3 then
					FusionHUDClassBar.frame.eclipseFrame:Hide();
					FusionHUDClassBar.frame.comboFrame:Show()
					FusionHUDClassBar.frame:SetWidth(FusionHUDClassBar.height);
					FusionHUDClassBar.frame:Show();
				elseif form == 5 and treeRank == 0 and moonRank == 1 then
					FusionHUDClassBar.frame.eclipseFrame:Show();
					FusionHUDClassBar.frame.comboFrame:Hide()
					FusionHUDClassBar.frame:SetWidth(FusionHUDClassBar.width);
					FusionHUDClassBar.frame:Show();
					if GetEclipseDirection() == "none" then
						FusionHUDClassBar.frame.eclipseFrame.arrowFrame.texture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\Eclipse Neutral", false);
					elseif GetEclipseDirection() == "moon" then
						FusionHUDClassBar.frame.eclipseFrame.arrowFrame.texture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\Eclipse Arrow", false);
						FusionHUDClassBar.frame.eclipseFrame.arrowFrame.texture:SetTexCoord(1,0,0,1);
					elseif GetEclipseDirection() == "sun" then
						FusionHUDClassBar.frame.eclipseFrame.arrowFrame.texture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\Eclipse Arrow", false);
						FusionHUDClassBar.frame.eclipseFrame.arrowFrame.texture:SetTexCoord(0,1,0,1);
					end
				end
				FusionHUDClassBar.frame:Show();
				return 
			end 
			local form = GetShapeshiftForm(false)
			local treeName, treeTex, treeTier, treeCol, treeRank = GetTalentInfo(3, 21, false, false, nil);
			local moonName, moonTex, moonTier, moonCol, moonRank = GetTalentInfo(1, 8, false, false, nil);
			if form == 3 then
				FusionHUDClassBar.frame.eclipseFrame:Hide();
				FusionHUDClassBar.frame.comboFrame:Show()
				FusionHUDClassBar.frame:SetWidth(FusionHUDClassBar.height);
				FusionHUDClassBar.frame:Show();
			elseif form == 5 and treeRank == 0 and moonRank == 1 then
				FusionHUDClassBar.frame.eclipseFrame:Show();
				FusionHUDClassBar.frame.comboFrame:Hide()
				FusionHUDClassBar.frame:SetWidth(FusionHUDClassBar.width);
				FusionHUDClassBar.frame:Show();
				if GetEclipseDirection() == "none" then
					FusionHUDClassBar.frame.eclipseFrame.arrowFrame.texture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\Eclipse Neutral", false);
				elseif GetEclipseDirection() == "moon" then
					FusionHUDClassBar.frame.eclipseFrame.arrowFrame.texture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\Eclipse Arrow", false);
					FusionHUDClassBar.frame.eclipseFrame.arrowFrame.texture:SetTexCoord(1,0,0,1);
				elseif GetEclipseDirection() == "sun" then
					FusionHUDClassBar.frame.eclipseFrame.arrowFrame.texture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\Eclipse Arrow", false);
					FusionHUDClassBar.frame.eclipseFrame.arrowFrame.texture:SetTexCoord(0,1,0,1);
				end
			else
				FusionHUDClassBar.frame:Hide()
			end
		end
	end
end
-- Minimap Button Functions (tooltips)
function FusionHUD:MinimapButtonOnEnter()
	GameTooltip:SetOwner(FusionHUDMinimapButtonFrame, "ANCHOR_TOPRIGHT");
	GameTooltip:ClearLines();
	GameTooltip:SetAlpha(1);
	if FusionHUDLockFrame:IsVisible() then
		GameTooltip:SetText("FusionHUD |cffff9933" .. GetAddOnMetadata("FusionHUD", "Version") .. "|r", 1, .9, 0,  1, 1);
		GameTooltip:AddLine("|cffff3333Please relock your bars using the FusionHUD Lock Button");
	else
		GameTooltip:SetText("FusionHUD |cffff9933" .. GetAddOnMetadata("FusionHUD", "Version") .. "|r", 1, .9, 0,  1, 1);
		GameTooltip:AddLine("|cff44ff44Click to open FusionHUD Options", 1,1,1,1,1);
	end
	GameTooltip:Show();
end

function FusionHUD:MinimapButtonOnLeave(...)
	GameTooltip:Hide();
end
--Deletes the last bar created
function FusionHUD:DeleteBar()
	local index = -1;
	table.foreach(FusionHUDBars, function(k,v) if index < v.barIndex then index = v.barIndex end end);
	if index then
		FusionHUDOptions.args.bars.args["bar" .. index] = nil;
		FusionHUDBars["bar" .. index].frame:Hide();
		FusionHUDBars["bar" .. index] = nil;
	else
		FusionHUDBars = {};
	end
end
--Sets global HUD transparency for context-sensitive transparency
function FusionHUD:Transparency()
	local localClass, englishClass = UnitClass("player");
	if FusionHUDModSettings.multiTransparency == false then
		FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.transparency);
	elseif UnitAffectingCombat("player") == 1 then
		FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.hiVis);
	elseif UnitDebuff("player", 1) ~= nil then
		FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.midVis);
	elseif UnitName("target") and UnitName("target") ~= "Unknown" then
		FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.midVis);
	elseif UnitHealth("player") ~= UnitHealthMax("player") then
		FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.midVis);
	elseif UnitPower("player")~= UnitPowerMax("player") then
		if englishClass == "WARRIOR" or englishClass == "DEATHKNIGHT" then
			if UnitPower("player") == 0 then
				FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.loVis);
			else
				FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.midVis);
			end
		else
			FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.midVis);
		end
	else
		FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.loVis);
	end
end
-------------------------------
-- Class Bar functions follow--
-------------------------------
function FusionHUD:BuildClassBarFrame()
	FusionHUDClassBar.barIndex = "Class";
	FusionHUDClassBar.frame = CreateFrame("Frame", nil, FusionHUDMasterFrame);
	FusionHUDClassBar.frame:SetPoint(FusionHUDClassBar.myPoint, "FusionHUDMasterFrame", FusionHUDClassBar.parentPoint, FusionHUDClassBar.offsetX, FusionHUDClassBar.offsetY);
	FusionHUDClassBar.frame:SetHeight(FusionHUDClassBar.height);
	FusionHUDClassBar.frame:SetWidth(FusionHUDClassBar.height * 6);
	FusionHUDClassBar.frame.backdropTexture = FusionHUDClassBar.frame:CreateTexture(nil, "BACKGROUND");
	FusionHUDClassBar.frame.backdropTexture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\whitebackdropbase", true);
	FusionHUDClassBar.frame.backdropTexture:ClearAllPoints();
	FusionHUDClassBar.frame.titleRegion = FusionHUDClassBar.frame:CreateTitleRegion();
	FusionHUDClassBar.frame.titleRegion:ClearAllPoints();
	if FusionHUDModSettings.classBarToggle then
		FusionHUDClassBar.frame:Show();
	else
		FusionHUDClassBar.frame:Hide();
	end
end

function FusionHUD:ClassSelector()
	local localizedClass, englishClass = UnitClass("player");
	local powerType, powerTypeString = UnitPowerType("player");
	if englishClass == "DEATHKNIGHT" then
		FusionHUD:BuildDKClassBar()
	elseif englishClass == "PALADIN" then
		FusionHUD:BuildPaladinClassBar()
	elseif englishClass == "WARLOCK" then
		FusionHUD:BuildWarlockClassBar()
	elseif englishClass == "ROGUE" then
		FusionHUD:BuildComboPointClassBar();
	elseif englishClass == "DRUID" then 
		FusionHUD:BuildEclipseBar();
		FusionHUD:BuildComboPointClassBar();
		if FusionHUDModSettings.classBarToggle == true then
			if powerTypeString ~= "ENERGY" then
				FusionHUDClassBar.frame.comboFrame:Hide();
			else
				FusionHUDClassBar.frame.comboFrame:Show();
			end
			if englishClass == "DRUID" then
				local name, tex, tier, col, rank = GetTalentInfo(3, 21, false, false, nil)
				if GetShapeshiftForm(false) == 5 and rank == 0 then 
					FusionHUDClassBar.frame.eclipseFrame:Show()
				else
					FusionHUDClassBar.frame.eclipseFrame:Hide()
				end
			end
		end
	end
end

function FusionHUD:BuildComboPointClassBar()
	FusionHUDClassBar.frame.comboFrame = FusionHUDComboPointObject:New()
	FusionHUDClassBar.frame.comboFrame.text:SetText(GetComboPoints("player", "target"));
	FusionHUD:RegisterEvent("UNIT_COMBO_POINTS");
end

function FusionHUD:BuildEclipseBar()
	FusionHUDClassBar.frame.eclipseFrame = FusionHUDEclipseObject:New();
	FusionHUDClassBar.frame.eclipseFrame:Show();
	FusionHUDClassBar.frame.eclipseFrame.arrowFrame:SetPoint("CENTER", FusionHUDClassBar.frame.eclipseFrame, "CENTER", (UnitPower("Player", 8)/100) * FusionHUDClassBar.width / 1.9, 0);
	FusionHUD:RegisterEvent("UNIT_POWER");
end

function FusionHUD:BuildDKClassBar()
	local i;
	FusionHUDClassBar.frame.runes = {};
	for i = 1,6 do
		FusionHUDClassBar.frame.runes[i] = FusionHUDRuneObject:New();
		FusionHUDClassBar.frame.runes[i].runeID = i;
		FusionHUDClassBar.frame.runes[i].texture:SetTexture(runeTextures[GetRuneType(i)]);
	end
	for i = 1,6 do
		if i == 1 then
			FusionHUDClassBar.frame.runes[i]:SetPoint("TOPLEFT", FusionHUDClassBar.frame, "TOPLEFT");
			FusionHUDClassBar.frame.runes[i]:SetPoint("BOTTOMLEFT", FusionHUDClassBar.frame, "BOTTOMLEFT");
		else
			FusionHUDClassBar.frame.runes[i]:SetPoint("TOPLEFT", FusionHUDClassBar.frame.runes[i-1], "TOPRIGHT");
			FusionHUDClassBar.frame.runes[i]:SetPoint("BOTTOMLEFT", FusionHUDClassBar.frame.runes[i-1], "BOTTOMRIGHT");
		end
	end
	FusionHUD:RegisterEvent("RUNE_POWER_UPDATE");
	FusionHUD:RegisterEvent("RUNE_TYPE_UPDATE");
end

function FusionHUD:BuildPaladinClassBar()
	FusionHUDClassBar.frame.powerBar = FusionHUDThreeCountObject:New();
	FusionHUDClassBar.frame.powerBar.fillTexture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\Three Count Fill Paladin", false);
	if UnitPower("player", 9) == 0 then
		FusionHUDClassBar.frame.powerBar.fillTexture:SetWidth(1);
	else
		FusionHUDClassBar.frame.powerBar.fillTexture:SetWidth(FusionHUDClassBar.width / 3 * UnitPower("player", 9));
	end
	FusionHUDClassBar.frame.powerBar.fillTexture:SetTexCoord(0, UnitPower("player", 9)/3, 1, 0);
	FusionHUD:RegisterEvent("UNIT_POWER");
end

function FusionHUD:BuildWarlockClassBar()
	FusionHUDClassBar.frame.powerBar = FusionHUDThreeCountObject:New();
	FusionHUDClassBar.frame.powerBar.fillTexture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\Three Count Fill Warlock", false);
	if UnitPower("player", 7) == 0 then
		FusionHUDClassBar.frame.powerBar.fillTexture:SetWidth(1);
	else
		FusionHUDClassBar.frame.powerBar.fillTexture:SetWidth(FusionHUDClassBar.width / 3 * UnitPower("player", 7));
	end 
	FusionHUDClassBar.frame.powerBar.fillTexture:SetTexCoord(0, UnitPower("player", 7)/3, 0, 1);
	FusionHUD:RegisterEvent("UNIT_POWER");
end
--Runes
function FusionHUD:RUNE_POWER_UPDATE(event, ...)
	local runeID = ...;
	local start, duration, runeReady = GetRuneCooldown(runeID);
	if runeReady == false then
		FusionHUDClassBar.frame.runes[runeID].texture:SetVertexColor(0.1,0.1,0.1);
		FusionHUDClassBar.frame.runes[runeID].cooldownFrame:Show();
		FusionHUDClassBar.frame.runes[runeID].cooldownFrame:SetScript('OnUpdate', function() FusionHUD:RuneCooldown(runeID) end);
	else
		FusionHUDClassBar.frame.runes[runeID].texture:SetVertexColor(1,1,1);
		FusionHUDClassBar.frame.runes[runeID].cooldownFrame:SetScript('OnUpdate', nil);
		FusionHUDClassBar.frame.runes[runeID].cooldownFrame:Hide();
	end
end

function FusionHUD:RUNE_TYPE_UPDATE(event, ...)
	local runeID = ...;
	local runeType = GetRuneType(runeID);
	FusionHUDClassBar.frame.runes[runeID].texture:SetTexture(runeTextures[runeType], false);
end
--Holy Power, Soul Shards, Eclipse Energy
function FusionHUD:UNIT_POWER(event, ...)
	local unit, powerType = ...;
	local localClass, englishClass = UnitClass("player");
	if unit == "player" then
		if powerType == "HOLY_POWER" and englishClass == "PALADIN" then
			if UnitPower("player", 9) == 0 then
				FusionHUDClassBar.frame.powerBar.fillTexture:SetWidth(1);
			else
				FusionHUDClassBar.frame.powerBar.fillTexture:SetWidth(FusionHUDClassBar.width / 3 * UnitPower("player", 9));
				FusionHUDClassBar.frame.powerBar.fillTexture:SetTexCoord(0, UnitPower("player", 9)/3, 0, 1);
			end
		elseif powerType == "SOUL_SHARDS" and englishClass == "WARLOCK" then
			if UnitPower("player", 7) == 0 then
				FusionHUDClassBar.frame.powerBar.fillTexture:SetWidth(1);
			else
				FusionHUDClassBar.frame.powerBar.fillTexture:SetWidth(FusionHUDClassBar.width / 3 * UnitPower("player", 7));
				FusionHUDClassBar.frame.powerBar.fillTexture:SetTexCoord(0, UnitPower("player", 7)/3, 0, 1);
			end				
		elseif powerType == "ECLIPSE" and englishClass == "DRUID" then
			FusionHUDClassBar.frame.eclipseFrame.arrowFrame:SetPoint("CENTER", FusionHUDClassBar.frame.eclipseFrame, "CENTER", (UnitPower("Player", 8)/100) * FusionHUDClassBar.width / 1.9, 0);
		end		
		
	end	
end

function FusionHUD:RuneCooldown(runeID)
	local nextUpdateClock =  GetTime();
	local start, duration, runeReady = GetRuneCooldown(runeID);
	if nextUpdateClock - FusionHUDClassBar.frame.runes[runeID].cooldownFrame.lastUpdateClock > 0.2 then
		FusionHUDClassBar.frame.runes[runeID].cooldownFrame.lastUpdateClock = nextUpdateClock
		if runeReady == false and math.ceil(duration - (nextUpdateClock - start)) >= 0 and math.ceil(duration - (nextUpdateClock - start)) <= duration then
			FusionHUDClassBar.frame.runes[runeID].cooldownFrame.text:SetFont("Interface\\AddOns\\FusionHUD\\Fonts\\DroidSans.ttf", math.floor(FusionHUDClassBar.frame:GetHeight() * .6), "OUTLINE");
			FusionHUDClassBar.frame.runes[runeID].cooldownFrame.text:SetText(math.ceil(duration-(nextUpdateClock-start)));
			FusionHUDClassBar.frame.runes[runeID].cooldownFrame.text:SetVertexColor(1,1,1,1);
		else
			FusionHUDClassBar.frame.runes[runeID].cooldownFrame.text:SetVertexColor(1,1,1,0);
		end
	end
end
--Combo Points
function FusionHUD:UNIT_COMBO_POINTS(event, ...)
	FusionHUDClassBar.frame.comboFrame.text:SetFont("Interface\\AddOns\\FusionHUD\\Fonts\\DroidSans.ttf", math.floor(FusionHUDClassBar.frame:GetHeight() * .8), "OUTLINE");
	FusionHUDClassBar.frame.comboFrame.text:SetText(GetComboPoints("player", "target"));
end

---------------------------
--End Class Bar functions--
---------------------------

--Handles bar unlocking and locking depending on HUD state
function FusionHUD:LockHandler(self)
	if self.barIndex ==  nil then
		return
	end
	if self.draggable == true then
		self.frame.titleRegion:SetAllPoints(self.frame);
		self.frame:SetBackdrop({bgFile = "Interface\\AddOns\\FusionHUD\\Graphics\\whitebackdropbase", tile = true, tilesize = self.width});
		self.frame:SetBackdropColor(0.8, 0.8, 0.8, 1);
		self.frame:Show();
		self.frame:SetAlpha(1);
		self.frame:EnableMouse(true);
		self.frame:SetMovable(true);
		if self.textIsDocked ~= nil and self.textIsDocked == false then
			self.frame.textFrame.titleRegion:SetAllPoints(self.frame.textFrame);
			self.frame.textFrame:SetBackdrop({bgFile = "Interface\\AddOns\\FusionHUD\\Graphics\\whitebackdropbase", tile = true, tilesize = 150});
			self.frame.textFrame:EnableMouse(true);
			self.frame.textFrame:SetMovable(true);
		end
		return
	else
		self.myPoint, self.parentFrame, self.parentPoint, self.offsetX, self.offsetY = self.frame:GetPoint();
		self.frame.titleRegion:ClearAllPoints();
		self.frame:SetBackdrop({nil});
		self.frame:EnableMouse(false);
		self.frame:SetMovable(false);
		FusionHUDLockFrame:Hide();
		InterfaceOptionsFrame_OpenToCategory(FusionHUD.optionsFrame);
		if self.textIsDocked ~= nil and self.textIsDocked == false then
			self.textPoint, self.textParent, self.textParentPoint, self.textOffsetX, self.textOffsetY = self.frame.textFrame:GetPoint();
			self.textParentName = tostring(self.textParent);
			self.frame.textFrame.titleRegion:ClearAllPoints();
			self.frame.textFrame:SetBackdrop({nil});
			self.frame.textFrame:EnableMouse(false);
			self.frame.textFrame:SetMovable(false);
		elseif self.textIsDocked ~= nil then
			self.frame.textFrame:SetPoint("TOP", self.frame, "BOTTOM");
			self.frame.textFrame.titleRegion:ClearAllPoints();
			self.frame.textFrame.backdropTexture:ClearAllPoints();
			self.frame.textFrame:EnableMouse(false);
			self.frame.textFrame:SetMovable(false);
		end
	end
end
